<?php

/**
 * Request handler for handling requests related to training & placement module
 *
 * @author Softpro India Pvt. Ltd
 */
class TrainingAndPlacement extends CI_Controller {

    public function __construct() {
        parent::__construct();
        $this->load->model('admin/TAndPManagement');
        $this->load->model('admin/LocationManagement');
        $this->load->model('admin/SessionManagement');
    }

    public function index() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $filteredRecords = array();
            $searchKeyword = '';
            if (isset($_POST['filterSubmitBtn'])) {
                $searchKeyword = addslashes(trim($this->input->post("searchKeyWord")));
                $selectedOrgCategory = trim($this->input->post("orgCategory"));
                $selectedOrgIndustry = trim($this->input->post("orgIndustry"));
                $filteredRecords = $this->TAndPManagement->getAllOrganizations($searchKeyword, $selectedOrgCategory, $selectedOrgIndustry)->result();
            }
            $viewData['key'] = $searchKeyword;
            $viewData['filteredRecords'] = $filteredRecords;
            $this->load->view('admin/trainingAndPlacement/organisations', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createOrganisation() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['states'] = $this->LocationManagement->getAllNonDeletedActiveStates()->result();
            $this->load->view('admin/trainingAndPlacement/createOrganisation', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function saveOrganisation() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('orgName', 'Organisation Name', 'trim|required', array('required' => 'Organisation Name Is Required.'));
            $this->form_validation->set_rules('orgCINNo', 'CIN Number', 'trim|required|min_length[21]|max_length[21]', array('required' => 'CIN Number Is Required.', 'min_length' => 'CIN Number Must Be Of 21 Character', 'max_length' => 'CIN Number Must Be Of 21 Character'));
            $this->form_validation->set_rules('orgMailingAddrLineOne', 'Address Line One', 'trim|required', array('required' => 'Address Line One Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrState', 'State', 'trim|required', array('required' => 'State Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrCity', 'City', 'trim|required', array('required' => 'City Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrZipCode', 'Zipcode', 'trim|required', array('required' => 'Zipcode Is Required.'));
            $this->form_validation->set_rules('orgMobile', 'Mobile Number', 'trim|required|numeric|min_length[10]|max_length[10]', array('required' => 'Mobile Is Required.'));
            $this->form_validation->set_rules('orgEmail', 'Email', 'trim|required', array('required' => 'Email Is Requireed.'));
            $this->form_validation->set_rules('orgCategory', 'Category', 'trim|required', array('required' => 'Category Is Required.'));
            $this->form_validation->set_rules('orgIndustry', 'Industry', 'trim|required', array('required' => 'Industry Is Required.'));
            $queryByOrgName = $this->TAndPManagement->getOrganizationInfoBy('', '', '', '', trim($this->input->post('orgName')));
            $orgInfoInfoByName = $queryByOrgName->result();
            $queryByCINNo = $this->TAndPManagement->getOrganizationInfoBy('', '', '', trim($this->input->post('orgCINNo')));
            $orgInfoInfoByCINNo = $queryByCINNo->result();
            $queryByEmail = $this->TAndPManagement->getOrganizationInfoBy('', trim($this->input->post('orgEmail')));
            $orgInfoInfoByEmail = $queryByEmail->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createOrganisation();
            } else if (sizeof($orgInfoInfoByName)) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This Name (" . trim($this->input->post('orgName')) . ") Already Exits! Please Enter Correct Organisation Name.");
                $this->createOrganisation();
            } else if (sizeof($orgInfoInfoByCINNo)) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This CIN. No. (" . trim($this->input->post('orgCINNo')) . ") Already Exits! Please Enter Correct CIN No..");
                $this->createOrganisation();
            } else if (sizeof($orgInfoInfoByEmail)) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This Email (" . trim($this->input->post('orgEmail')) . ") Already Exits! Please Enter Correct Email.");
                $this->createOrganisation();
            } else {
                if (!empty($_FILES['orgLogo']['name'])) {
                    $corpLogo['upload_path'] = './assets/admin/uploads/trainingAndPlacement/orgLogos/';
                    $corpLogo['allowed_types'] = 'jpeg|jpg|png';
                    $corpLogo['max_size'] = '500';
                    $corpLogo['file_ext_tolower'] = TRUE;
                    $corpLogo['encrypt_name'] = TRUE;
                    $this->load->library('upload', $corpLogo);
                    if (!$this->upload->do_upload('orgLogo')) {
                        $this->session->set_flashdata('errorMessage', "Organisation Logo Can Not Be Uploaded As It Violates Upload File Criteria. File Size 500 KB Max. & Allowed Extension: Only *.png,*.jpg,*.jpeg.");
                        $this->createOrganisation();
                    } else {
                        $logo = 'assets/admin/uploads/trainingAndPlacement/orgLogos/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $logo = 'assets/admin/uploads/trainingAndPlacement/orgLogos/logonotavilable.png';
                }
                $orgInfo = array(
                    'pla_org_name' => addslashes(trim($this->input->post('orgName'))),
                    'pla_org_short_name' => addslashes(trim($this->input->post('orgShortName'))),
                    'pla_org_cin_no' => addslashes(trim($this->input->post('orgCINNo'))),
                    'pla_org_mobile' => trim($this->input->post('orgMobile')),
                    'pla_org_landline' => trim($this->input->post('orgLandline')),
                    'pla_org_email' => trim($this->input->post('orgEmail')),
                    'pla_org_size' => trim($this->input->post('orgSize')),
                    'pla_org_headquarter' => addslashes(trim($this->input->post('orgHead'))),
                    'pla_org_cmm_level' => trim($this->input->post('orgCMMLevel')),
                    'pla_org_vision_msg' => addslashes(trim($this->input->post('orgVisionMsg'))),
                    'pla_org_website_link' => trim($this->input->post('orgWebLink')),
                    'pla_org_linkedin_link' => trim($this->input->post('orgLinkedInLink')),
                    'pla_org_google_map_location' => trim($this->input->post('orgGMap')),
                    'pla_org_logo' => $logo,
                    'pla_org_type' => trim($this->input->post('orgCategory')),
                    'pla_org_industry' => trim($this->input->post('orgIndustry')),
                    'pla_org_foundation_date' => ($this->input->post('orgFoundationDate') == "" ) ? NULL : date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('orgFoundationDate')))),
                    'pla_org_added_by' => $this->session->userdata("adminData")["smember_id"],
                    'pla_org_added_on' => date("Y-m-d H:i:s"),
                    'pla_org_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'pla_org_updated_on' => date("Y-m-d H:i:s"),
                    'pla_org_address_line_one' => addslashes(trim($this->input->post('orgMailingAddrLineOne'))),
                    'pla_org_address_line_two' => addslashes(trim($this->input->post('orgMailingAddrLineTwo'))),
                    'pla_org_address_line_three' => addslashes(trim($this->input->post('orgMailingAddrLineThree'))),
                    'pla_org_zipcode' => trim($this->input->post('orgMailingAddrZipCode')),
                    'pla_org_state' => trim($this->input->post('orgMailingAddrState')),
                    'pla_org_city' => trim($this->input->post('orgMailingAddrCity')),
                );
                $this->db->trans_start();
                $pla_org_id = $this->TAndPManagement->createNewOrganization($orgInfo);
                if ($pla_org_id) {
                    $this->db->trans_complete();
                    $this->session->set_flashdata('successMessage', 'Organisation Details Saved Successfully.');
                    redirect("admin/TrainingAndPlacement/");
                } else {
                    $this->session->set_flashdata('successMessage', 'An Error Occured While Adding Organisation Details. Try Again.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function editOrganisation($pla_org_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['states'] = $this->LocationManagement->getAllNonDeletedActiveStates()->result();
            $viewData['orgInfo'] = $this->TAndPManagement->getOrganizationInfoBy($pla_org_id)->result()[0];
            $this->load->view('admin/trainingAndPlacement/editOrganisation', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateOrganisation() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('orgName', 'Organisation Name', 'trim|required', array('required' => 'Organisation Name Is Required.'));
            $this->form_validation->set_rules('orgCINNo', 'CIN Number', 'trim|required|min_length[21]|max_length[21]', array('required' => 'CIN Number Is Required.', 'min_length' => 'CIN Number Must Be Of 21 Character', 'max_length' => 'CIN Number Must Be Of 21 Character'));
            $this->form_validation->set_rules('orgMailingAddrLineOne', 'Address Line One', 'trim|required', array('required' => 'Address Line One Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrState', 'State', 'trim|required', array('required' => 'State Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrCity', 'City', 'trim|required', array('required' => 'City Is Required.'));
            $this->form_validation->set_rules('orgMailingAddrZipCode', 'Zipcode', 'trim|required', array('required' => 'Zipcode Is Required.'));
            $this->form_validation->set_rules('orgMobile', 'Mobile Number', 'trim|required|numeric|min_length[10]|max_length[10]', array('required' => 'Mobile Is Required.'));
            $this->form_validation->set_rules('orgEmail', 'Email', 'trim|required', array('required' => 'Email Is Requireed.'));
            $this->form_validation->set_rules('orgCategory', 'Category', 'trim|required', array('required' => 'Category Is Required.'));
            $this->form_validation->set_rules('orgIndustry', 'Industry', 'trim|required', array('required' => 'Industry Is Required.'));
            $pla_org_id = $this->input->post('pla_org_id');
            $queryByOrgName = $this->TAndPManagement->isOrganisationDetailsSafeToUpdate($pla_org_id, trim($this->input->post('orgName')));
            $orgInfoInfoByName = $queryByOrgName;
            $queryByCINNo = $this->TAndPManagement->isOrganisationDetailsSafeToUpdate($pla_org_id, '', trim($this->input->post('orgCINNo')), '');
            $orgInfoInfoByCINNo = $queryByCINNo;
            $queryByEmail = $this->TAndPManagement->isOrganisationDetailsSafeToUpdate($pla_org_id, '', '', trim($this->input->post('orgEmail')));
            $orgInfoInfoByEmail = $queryByEmail;
            if ($this->form_validation->run() == FALSE) {
                $this->editOrganisation($pla_org_id);
            } else if (!$orgInfoInfoByName) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This Name (" . trim($this->input->post('orgName')) . ") Already Exits! Please Enter Correct Organisation Name.");
                $this->editOrganisation($pla_org_id);
            } else if (!$orgInfoInfoByCINNo) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This CIN. No. (" . trim($this->input->post('orgCINNo')) . ") Already Exits! Please Enter Correct CIN No..");
                $this->editOrganisation($pla_org_id);
            } else if (!$orgInfoInfoByEmail) {
                $this->session->set_flashdata('errorMessage', "A Organisation Record With This Email (" . trim($this->input->post('orgEmail')) . ") Already Exits! Please Enter Correct Email.");
                $this->editOrganisation($pla_org_id);
            } else {
                $this->load->library('upload');
                if (!empty($_FILES['orgLogo']['name'])) {
                    $studentImg['upload_path'] = './assets/admin/uploads/trainingAndPlacement/orgLogos';
                    $studentImg['allowed_types'] = 'jpeg|jpg|png';
                    $studentImg['max_size'] = '500';
                    $studentImg['file_ext_tolower'] = TRUE;
                    $studentImg['encrypt_name'] = TRUE;
                    $this->upload->initialize($studentImg);
                    if (!$this->upload->do_upload('orgLogo')) {
                        $this->session->set_flashdata('errorMessage', "Organisation Logo Can Not Be Uploaded As It Violates Upload File Criteria. File Size 500 KB Max. & Allowed Extension: Only *.png,*.jpg,*.jpeg.");
                        $this->editOrganisation($pla_org_id);
                    } else {
                        if (file_exists("./assets/admin/uploads/trainingAndPlacement/orgLogos/" . basename($this->input->post('old_logo'))) && (basename($this->input->post('old_logo')) != "logonotavilable.png")) {
                            unlink("./assets/admin/uploads/trainingAndPlacement/orgLogos/" . basename($this->input->post('old_logo')));
                        }
                        $logo = 'assets/admin/uploads/trainingAndPlacement/orgLogos/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $logo = $this->input->post('old_logo');
                }
                $orgInfo = array(
                    'pla_org_name' => addslashes(trim($this->input->post('orgName'))),
                    'pla_org_short_name' => addslashes(trim($this->input->post('orgShortName'))),
                    'pla_org_cin_no' => addslashes(trim($this->input->post('orgCINNo'))),
                    'pla_org_mobile' => trim($this->input->post('orgMobile')),
                    'pla_org_landline' => trim($this->input->post('orgLandline')),
                    'pla_org_email' => trim($this->input->post('orgEmail')),
                    'pla_org_size' => trim($this->input->post('orgSize')),
                    'pla_org_headquarter' => addslashes(trim($this->input->post('orgHead'))),
                    'pla_org_cmm_level' => trim($this->input->post('orgCMMLevel')),
                    'pla_org_vision_msg' => addslashes(trim($this->input->post('orgVisionMsg'))),
                    'pla_org_website_link' => trim($this->input->post('orgWebLink')),
                    'pla_org_linkedin_link' => trim($this->input->post('orgLinkedInLink')),
                    'pla_org_google_map_location' => trim($this->input->post('orgGMap')),
                    'pla_org_logo' => $logo,
                    'pla_org_type' => trim($this->input->post('orgCategory')),
                    'pla_org_industry' => trim($this->input->post('orgIndustry')),
                    'pla_org_foundation_date' => ($this->input->post('orgFoundationDate') == "" ) ? NULL : date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('orgFoundationDate')))),
                    'pla_org_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'pla_org_updated_on' => date("Y-m-d H:i:s"),
                    'pla_org_address_line_one' => addslashes(trim($this->input->post('orgMailingAddrLineOne'))),
                    'pla_org_address_line_two' => addslashes(trim($this->input->post('orgMailingAddrLineTwo'))),
                    'pla_org_address_line_three' => addslashes(trim($this->input->post('orgMailingAddrLineThree'))),
                    'pla_org_zipcode' => trim($this->input->post('orgMailingAddrZipCode')),
                    'pla_org_state' => trim($this->input->post('orgMailingAddrState')),
                    'pla_org_city' => trim($this->input->post('orgMailingAddrCity')),
                    'pla_org_id' => $pla_org_id
                );
                $this->db->trans_start();
                $pla_org_id = $this->TAndPManagement->updateOrganisationInfo($orgInfo);
                if ($pla_org_id) {
                    $this->db->trans_complete();
                    $this->session->set_flashdata('successMessage', 'Organisation Details Updated Successfully.');
                    redirect("admin/TrainingAndPlacement/");
                } else {
                    $this->session->set_flashdata('successMessage', 'An Error Occured While Updating Organisation Details. Try Again.');
                    redirect("admin/TrainingAndPlacement/editOrganisation/" . $pla_org_id);
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function toggleOrganisationStatus($pla_org_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $orgUpdateData = array(
                'pla_org_id' => $pla_org_id,
                'pla_org_updated_on' => date("Y-m-d H:i:s"),
                'pla_org_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'pla_org_active_status' => $toUpdateStatus
            );
            if ($this->TAndPManagement->updateOrganisationInfo($orgUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Organisation Status Updated Successfully.');
                redirect("admin/TrainingAndPlacement/");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Organisation Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteOrganisation($pla_org_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $orgUpdateData = array(
                'pla_org_id' => $pla_org_id,
                'pla_org_updated_on' => date("Y-m-d H:i:s"),
                'pla_org_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'pla_org_delete_status' => $toUpdateStatus
            );
            if ($this->TAndPManagement->updateOrganisationInfo($orgUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Organisation Details Deleted Stattus Successfully.');
                redirect("admin/TrainingAndPlacement/");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Deleting Organisation Details. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function getOrganisationFullInfo() {
        $pla_org_id = $_POST['pla_org_id'];
        $org = $this->TAndPManagement->getOrganizationInfoBy($pla_org_id)->result()[0];
        $responseData = array(
            'csrfName' => $this->security->get_csrf_token_name(),
            'csrfHash' => $this->security->get_csrf_hash(),
            'pla_org_id' => $org->pla_org_id,
            'pla_org_name' => stripslashes($org->pla_org_name),
            'pla_org_short_name' => stripslashes($org->pla_org_short_name),
            'pla_org_cin_no' => stripslashes($org->pla_org_cin_no),
            'pla_org_address' => stripslashes($org->pla_org_address_line_one . "," . (($org->pla_org_address_line_two == "") ? "" : $org->pla_org_address_line_two . ", ") . (($org->pla_org_address_line_three == "") ? "" : $org->pla_org_address_line_three . ", ") . $org->city_name . ", " . $org->state_name . "-" . $org->pla_org_zipcode),
            'pla_org_logo' => ($org->pla_org_logo == "" ) ? "N/A" : site_url(stripslashes($org->pla_org_logo)),
            'pla_org_foundation_date' => ($org->pla_org_foundation_date == "" ) ? "N/A" : stripslashes($org->pla_org_foundation_date),
            'pla_org_website_link' => ($org->pla_org_website_link == "" ) ? "N/A" : $org->pla_org_website_link,
            'pla_org_linkedin_link' => ($org->pla_org_linkedin_link == "" ) ? "N/A" : $org->pla_org_linkedin_link,
            'pla_org_mobile' => stripslashes($org->pla_org_mobile),
            'pla_org_email' => stripslashes($org->pla_org_email),
            'pla_org_vision_msg' => ($org->pla_org_vision_msg == "" ) ? "N/A" : stripslashes($org->pla_org_vision_msg),
            'pla_org_headquarter' => ($org->pla_org_headquarter == "" ) ? "N/A" : stripslashes($org->pla_org_headquarter),
            'pla_org_google_map_location' => ($org->pla_org_google_map_location == "" ) ? "N/A" : $org->pla_org_google_map_location,
            'pla_org_landline' => ($org->pla_org_landline == "" ) ? "N/A" : stripslashes($org->pla_org_landline),
            'pla_org_size' => ($org->pla_org_size == "" ) ? "N/A" : stripslashes($org->pla_org_size),
            'pla_org_type' => stripslashes($org->pla_org_type),
            'pla_org_industry' => stripslashes($org->pla_org_industry),
            'pla_org_cmm_level' => ($org->pla_org_cmm_level == "" ) ? "N/A" : stripslashes($org->pla_org_cmm_level),
            'pla_org_added_by' => stripslashes($org->addedByAdmin),
            'pla_org_updated_by' => stripslashes($org->updatedByAdmin),
            'pla_org_added_on' => date('d-m-Y h:i:s A', strtotime($org->pla_org_added_on)),
            'pla_org_updated_on' => date('d-m-Y h:i:s A', strtotime($org->pla_org_updated_on))
        );
        echo json_encode($responseData);
    }

    /* Request Handlers For Job Offers & Related Stuffs */

    public function jobOffers() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $selectedSession = "";
            $filteredRecords = array();
            if (isset($_POST['filterSubmitBtn'])) {
                $selectedSession = $this->input->post('session_id');
                $searchKeyword = addslashes(trim($this->input->post("searchKeyWord")));
                $selected_pla_org_id = $this->input->post('pla_org_id');
                $selectedStartDate = (trim($this->input->post("startDate")) == "") ? "" : date("Y-m-d 00:00:00", strtotime(str_replace('/', '-', trim($this->input->post("startDate")))));
                $selectedEndDate = (trim($this->input->post("endDate")) == "") ? "" : date("Y-m-d 23:59:59", strtotime(str_replace('/', '-', trim($this->input->post("endDate")))));
                $filteredRecords = $this->TAndPManagement->getAlljobOffers($searchKeyword, $selected_pla_org_id, $selectedSession, "T", "F", $selectedStartDate, $selectedEndDate)->result();
            } else {
                $selectedSession = $this->SessionManagement->getCurrentSessionInfo()->result()[0]->session_id;
                $filteredRecords = $this->TAndPManagement->getAlljobOffers("", "", $selectedSession, "", "F", "", "")->result();
            }
            $viewData['selectedSession'] = $selectedSession;
            $viewData["sessions"] = $this->SessionManagement->getNonDeletedActiveSessions()->result();
            $viewData['organisations'] = $this->TAndPManagement->getAllOrganizations()->result();
            $viewData['filteredRecords'] = $filteredRecords;
            $this->load->view('admin/trainingAndPlacement/jobOffers', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createJobOffer() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData["sessions"] = $this->SessionManagement->getNonDeletedActiveSessions()->result();
            $viewData['organisations'] = $this->TAndPManagement->getAllOrganizations('', '', '', 'T', 'F')->result();
            $this->load->view('admin/trainingAndPlacement/createJobOffer', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function saveNewJobOffer() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('session_id', 'Session', 'trim|required', array('required' => 'Session Is Required.'));
            $this->form_validation->set_rules('pla_org_id', 'Organisation', 'trim|required', array('required' => 'Organisation Is Required.',));
            $this->form_validation->set_rules('jobTitle', 'Job Title', 'trim|required', array('required' => 'Job Title Is Required.'));
            $check = $this->TAndPManagement->getJobOfferInfoBy('', trim($this->input->post('pla_org_id')), trim($this->input->post('jobTitle')), trim($this->input->post('session_id')))->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createJobOffer();
            } else if (sizeof($check)) {
                $this->session->set_flashdata('errorMessage', "A Job Offer Record With This Title (" . trim($this->input->post('jobTitle')) . "),  Already Exits! Please Enter Correct Job Title.");
                $this->createJobOffer();
            } else {
                $this->load->library('upload');
                if (!empty($_FILES['offerFile']['name'])) {
                    $jobOfferLetterConfig['upload_path'] = './assets/admin/uploads/trainingAndPlacement/';
                    $jobOfferLetterConfig['allowed_types'] = 'pdf';
                    $jobOfferLetterConfig['max_size'] = '2048';
                    $jobOfferLetterConfig['file_ext_tolower'] = TRUE;
                    $jobOfferLetterConfig['encrypt_name'] = TRUE;
                    $this->upload->initialize($jobOfferLetterConfig);
                    if (!$this->upload->do_upload('offerFile')) {
                        $this->session->set_flashdata('errorMessage', "Offer Letter Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->createJobOffer();
                    } else {
                        $offerLetter = 'assets/admin/uploads/trainingAndPlacement/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $offerLetter = '';
                }
                if (!empty($_FILES['resultFile']['name'])) {
                    $selectionResultLetterConfig['upload_path'] = './assets/admin/uploads/trainingAndPlacement/';
                    $selectionResultLetterConfig['allowed_types'] = 'pdf';
                    $selectionResultLetterConfig['max_size'] = '2048';
                    $selectionResultLetterConfig['file_ext_tolower'] = TRUE;
                    $selectionResultLetterConfig['encrypt_name'] = TRUE;
                    $this->upload->initialize($selectionResultLetterConfig);
                    if (!$this->upload->do_upload('resultFile')) {
                        $this->session->set_flashdata('errorMessage', "Selection Result Letter Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->createJobOffer();
                    } else {
                        $selecttionResultLetter = 'assets/admin/uploads/trainingAndPlacement/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $selecttionResultLetter = '';
                }
                $jobOfferInfo = array(
                    'pjo_title' => addslashes(trim($this->input->post('jobTitle'))),
                    'pla_org_id' => addslashes(trim($this->input->post('pla_org_id'))),
                    'session_id' => addslashes(trim($this->input->post('session_id'))),
                    'pjo_criteria' => trim($this->input->post('offerCriteria')),
                    'pjo_file' => $offerLetter,
                    'pjo_result_file' => $selecttionResultLetter,
                    'pjo_no_of_vacancy' => trim($this->input->post('vacancy')),
                    'pjo_package' => trim($this->input->post('offerPackage')),
                    'pjo_added_by' => $this->session->userdata("adminData")["smember_id"],
                    'pjo_added_on' => date("Y-m-d H:i:s"),
                    'pjo_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'pjo_updated_on' => date("Y-m-d H:i:s"),
                );
                $this->db->trans_start();
                $pjo_id = $this->TAndPManagement->createNewJobOffer($jobOfferInfo);
                if ($pjo_id) {
                    $this->db->trans_complete();
                    $this->session->set_flashdata('successMessage', 'Job Offer Created Saved Successfully.');
                    redirect("admin/TrainingAndPlacement/jobOffers");
                } else {
                    $this->session->set_flashdata('successMessage', 'An Error Occured While Creating Job Offer. Try Again.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function editJobOffer($pjo_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData["sessions"] = $this->SessionManagement->getNonDeletedActiveSessions()->result();
            $viewData['organisations'] = $this->TAndPManagement->getAllOrganizations()->result();
            $viewData['jobOffer'] = $this->TAndPManagement->getJobOfferInfoBy($pjo_id)->result()[0];
            $this->load->view('admin/trainingAndPlacement/editJobOffer', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateJobOffer() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('session_id', 'Session', 'trim|required', array('required' => 'Session Is Required.'));
            $this->form_validation->set_rules('pla_org_id', 'Organisation', 'trim|required', array('required' => 'Organisation Is Required.',));
            $this->form_validation->set_rules('jobTitle', 'Job Title', 'trim|required', array('required' => 'Job Title Is Required.'));
            $pjo_id = $this->input->post('pjo_id');
            if ($this->form_validation->run() == FALSE) {
                $this->editJobOffer($pjo_id);
            } else {
                $this->load->library('upload');
                if (!empty($_FILES['offerFile']['name'])) {
                    $jobOfferLetterConfig['upload_path'] = './assets/admin/uploads/trainingAndPlacement/';
                    $jobOfferLetterConfig['allowed_types'] = 'pdf';
                    $jobOfferLetterConfig['max_size'] = '2048';
                    $jobOfferLetterConfig['file_ext_tolower'] = TRUE;
                    $jobOfferLetterConfig['encrypt_name'] = TRUE;
                    $this->upload->initialize($jobOfferLetterConfig);
                    if (!$this->upload->do_upload('offerFile')) {
                        $this->session->set_flashdata('errorMessage', "Offer Letter Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->editJobOffer($pjo_id);
                    } else {
                        if (file_exists("./" . $this->input->post("oldFile"))) {
                            unlink("./" . $this->input->post("oldFile"));
                        }
                        $offerLetter = 'assets/admin/uploads/trainingAndPlacement/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $offerLetter = $this->input->post('oldFile');
                }
                if (!empty($_FILES['resultFile']['name'])) {
                    $selectionResultLetterConfig['upload_path'] = './assets/admin/uploads/trainingAndPlacement/';
                    $selectionResultLetterConfig['allowed_types'] = 'pdf';
                    $selectionResultLetterConfig['max_size'] = '2048';
                    $selectionResultLetterConfig['file_ext_tolower'] = TRUE;
                    $selectionResultLetterConfig['encrypt_name'] = TRUE;
                    $this->upload->initialize($selectionResultLetterConfig);
                    if (!$this->upload->do_upload('resultFile')) {
                        $this->session->set_flashdata('errorMessage', "Selection Result Letter Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->editJobOffer($pjo_id);
                    } else {
                        if (file_exists("./" . $this->input->post("oldResultFile"))) {
                            unlink("./" . $this->input->post("oldResultFile"));
                        }
                        $selecttionResultLetter = 'assets/admin/uploads/trainingAndPlacement/' . $this->upload->data()['file_name'];
                    }
                } else {
                    $selecttionResultLetter = $this->input->post('oldResultFile');
                }
                $jobOfferInfo = array(
                    'pjo_title' => addslashes(trim($this->input->post('jobTitle'))),
                    'pla_org_id' => addslashes(trim($this->input->post('pla_org_id'))),
                    'session_id' => addslashes(trim($this->input->post('session_id'))),
                    'pjo_criteria' => trim($this->input->post('offerCriteria')),
                    'pjo_file' => $offerLetter,
                    'pjo_result_file' => $selecttionResultLetter,
                    'pjo_no_of_vacancy' => trim($this->input->post('vacancy')),
                    'pjo_package' => trim($this->input->post('offerPackage')),
                    'pjo_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'pjo_updated_on' => date("Y-m-d H:i:s"),
                    'pjo_id' => $pjo_id
                );

                $this->db->trans_start();
                $pjo_id = $this->TAndPManagement->updateJobOffer($jobOfferInfo);
                if ($pjo_id) {
                    $this->db->trans_complete();
                    $this->session->set_flashdata('successMessage', 'Job Offer Details Updated Successfully.');
                    redirect("admin/TrainingAndPlacement/jobOffers");
                } else {
                    $this->session->set_flashdata('successMessage', 'An Error Occured While Updating Job Offer Details. Try Again.');
                    redirect("admin/TrainingAndPlacement/editJobOffer/" . $pjo_id);
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function togglejobOfferStatus($pjo_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $jobOfferUpdateData = array(
                'pjo_id' => $pjo_id,
                'pjo_updated_on' => date("Y-m-d H:i:s"),
                'pjo_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'pjo_active_status' => $toUpdateStatus
            );
            if ($this->TAndPManagement->updateJobOffer($jobOfferUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Job Offer Status Updated Successfully.');
                redirect("admin/TrainingAndPlacement/jobOffers");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Job Offer Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteJobOffers($pjo_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $jobOfferUpdateData = array(
                'pjo_id' => $pjo_id,
                'pjo_updated_on' => date("Y-m-d H:i:s"),
                'pjo_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'pjo_delete_status' => "T"
            );
            if ($this->TAndPManagement->updateJobOffer($jobOfferUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Job Offer Deleted Successfully.');
                redirect("admin/TrainingAndPlacement/jobOffers");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Deleting Job Offer. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

}
